/***********************************************************
Copyright 1999,2001,2008 by Tadayoshi Funaba, Tokyo, Japan. tadf@funaba.org

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Tadayoshi Funaba
not be used in advertising or publicity pertaining to distribution
of the software without specific, written prior permission.

TADAYOSHI FUNABA DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL TADAYOSHI FUNABA BE LIABLE FOR ANY SPECIAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

$Id: getdatemodule.c,v 1.4 2008-11-12 19:07:37+09 tadf Exp $

******************************************************************/

#include <Python.h>
#include "getdate.h"

static PyObject *
getdate_getdate(PyObject *self, PyObject *args)
{
    char *str;
    int argc, len;
    time_t time, when;

    if ((argc = PyTuple_Size(args)) == 1) {
	if (!PyArg_ParseTuple(args, "s#", &str, &len))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "s#l", &str, &len, &time))
	    return NULL;
    }
    if (strlen(str) < len) {
	char *d, *s, *e;

	d = alloca(len + 1);
	memcpy(d, str, len + 1);
	s = d;
	e = d + len;
	while (s < e) {
	    if (*s == '\0')
		*s = ' ';
	    s++;
	}
	str = d;
    }
    when = get_date(str, argc == 2 ? &time : NULL);
    if (when == -1) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("i", when);
}

static PyMethodDef GetdateMethods[] = {
    { "getdate", getdate_getdate, 1 },
    { NULL, NULL }
};

PyMODINIT_FUNC
initgetdate(void)
{
    PyObject *mod, *dict;

    mod = Py_InitModule("getdate", GetdateMethods);

    if (mod == NULL)
      return;

    dict = PyModule_GetDict(mod);
    (void)PyDict_SetItemString
	(dict, "__version__",
	 Py_BuildValue
	 ("s", "$Revision: 1.4 $"));
}
