/***********************************************************
Copyright 1997,1999,2008 by Tadayoshi Funaba, Tokyo, Japan. tadf@funaba.org

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Tadayoshi Funaba
not be used in advertising or publicity pertaining to distribution
of the software without specific, written prior permission.

TADAYOSHI FUNABA DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL TADAYOSHI FUNABA BE LIABLE FOR ANY SPECIAL,
INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

$Id: sdnmodule.c,v 1.5 2008-11-12 19:07:37+09 tadf Exp $

******************************************************************/

#include <Python.h>
#include "sdncal.h"

#define CHECK_RANGE(v, m)			\
{						\
    if ((v) < (m)) {				\
	PyErr_SetString				\
	    (PyExc_ValueError,			\
	     "out of range");			\
	return NULL;				\
    }						\
}

#define CHECK_DOMAIN(v, m)			\
{						\
    if ((v) < (m)) {				\
	PyErr_SetString				\
	    (PyExc_ValueError,			\
	     "out of domain");			\
	return NULL;				\
    }						\
}

#define CHECK_DOMAIN2(v, m, n)			\
{						\
    if ((v) < (m) || (v) > (n)) {		\
	PyErr_SetString				\
	    (PyExc_ValueError,			\
	     "out of domain");			\
	return NULL;				\
    }						\
}

static PyObject *
sdn_SdnToGregorian(PyObject *self, PyObject *args)
{
    long sdn;
    int y, m, md;

    if (!PyArg_ParseTuple(args, "l", &sdn))
	return NULL;
    CHECK_DOMAIN(sdn, 1);
    SdnToGregorian(sdn, &y, &m, &md);
    return Py_BuildValue("(iii)", y, m, md);
}

static PyObject *
sdn_GregorianToSdn(PyObject *self, PyObject *args)
{
    int y, m, md, y2, m2, md2;
    long sdn;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iii)", &y, &m, &md))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &y, &m, &md))
	    return NULL;
    }
    CHECK_DOMAIN(y, -4714);
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN2(md, 1, 31);
    sdn = GregorianToSdn(y, m, md);
    CHECK_RANGE(sdn, 1);
    SdnToGregorian(sdn, &y2, &m2, &md2);
    if (y != y2 || m != m2 || md != md2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("l", sdn);
}

static PyObject *
sdn_SdnToJulian(PyObject *self, PyObject *args)
{
    long sdn;
    int y, m, md;

    if (!PyArg_ParseTuple(args, "l", &sdn))
	return NULL;
    CHECK_DOMAIN(sdn, 1);
    SdnToJulian(sdn, &y, &m, &md);
    return Py_BuildValue("(iii)", y, m, md);
}

static PyObject *
sdn_JulianToSdn(PyObject *self, PyObject *args)
{
    int y, m, md, y2, m2, md2;
    long sdn;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iii)", &y, &m, &md))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &y, &m, &md))
	    return NULL;
    }
    CHECK_DOMAIN(y, -4713);
    CHECK_DOMAIN2(m, 1, 12);
    CHECK_DOMAIN2(md, 1, 31);
    sdn = JulianToSdn(y, m, md);
    CHECK_RANGE(sdn, 1);
    SdnToJulian(sdn, &y2, &m2, &md2);
    if (y != y2 || m != m2 || md != md2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("l", sdn);
}

static PyObject *
sdn_SdnToJewish(PyObject *self, PyObject *args)
{
    long sdn;
    int y, m, md;

    if (!PyArg_ParseTuple(args, "l", &sdn))
	return NULL;
    CHECK_DOMAIN(sdn, 347998);
    SdnToJewish(sdn, &y, &m, &md);
    return Py_BuildValue("(iii)", y, m, md);
}

static PyObject *
sdn_JewishToSdn(PyObject *self, PyObject *args)
{
    int y, m, md, y2, m2, md2;
    long sdn;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iii)", &y, &m, &md))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &y, &m, &md))
	    return NULL;
    }
    CHECK_DOMAIN(y, 1);
    CHECK_DOMAIN2(m, 1, 13);
    CHECK_DOMAIN2(md, 1, 30);
    sdn = JewishToSdn(y, m, md);
    CHECK_RANGE(sdn, 1); /* 347998 */
    SdnToJewish(sdn, &y2, &m2, &md2);
    if (y != y2 || m != m2 || md != md2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("l", sdn);
}

static PyObject *
sdn_SdnToFrench(PyObject *self, PyObject *args)
{
    long sdn;
    int y, m, md;

    if (!PyArg_ParseTuple(args, "l", &sdn))
	return NULL;
    CHECK_DOMAIN2(sdn, 2375840, 2380952);
    SdnToFrench(sdn, &y, &m, &md);
    return Py_BuildValue("(iii)", y, m, md);
}

static PyObject *
sdn_FrenchToSdn(PyObject *self, PyObject *args)
{
    int y, m, md, y2, m2, md2;
    long sdn;

    if (PyTuple_Size(args) == 1) {
	if (!PyArg_ParseTuple(args, "(iii)", &y, &m, &md))
	    return NULL;
    } else {
	if (!PyArg_ParseTuple(args, "iii", &y, &m, &md))
	    return NULL;
    }
    CHECK_DOMAIN(y, 1);
    CHECK_DOMAIN2(m, 1, 13);
    CHECK_DOMAIN2(md, 1, 30);
    sdn = FrenchToSdn(y, m, md);
    CHECK_RANGE(sdn, 1);
    SdnToFrench(sdn, &y2, &m2, &md2);
    if (y != y2 || m != m2 || md != md2) {
	PyErr_SetString
	    (PyExc_ValueError,
	     "invalid date");
	return NULL;
    }
    return Py_BuildValue("l", sdn);
}

static PyObject *
sdn_DayOfWeek(PyObject *self, PyObject *args)
{
    long sdn;
    int wd;

    if (!PyArg_ParseTuple(args, "l", &sdn))
	return NULL;
    wd = DayOfWeek(sdn);
    return Py_BuildValue("i", wd);
}

static PyMethodDef SdnMethods[] = {
    { "SdnToGregorian", sdn_SdnToGregorian, 1 },
    { "GregorianToSdn", sdn_GregorianToSdn, 1 },
    { "SdnToJulian", sdn_SdnToJulian, 1 },
    { "JulianToSdn", sdn_JulianToSdn, 1 },
    { "SdnToJewish", sdn_SdnToJewish, 1 },
    { "JewishToSdn", sdn_JewishToSdn, 1 },
    { "SdnToFrench", sdn_SdnToFrench, 1 },
    { "FrenchToSdn", sdn_FrenchToSdn, 1 },
    { "DayOfWeek", sdn_DayOfWeek, 1 },
    { NULL, NULL }
};

PyMODINIT_FUNC
initsdn(void)
{
    PyObject *mod, *dict;

    mod = Py_InitModule("sdn", SdnMethods);

    if (mod == NULL)
      return;

    dict = PyModule_GetDict(mod);
    (void)PyDict_SetItemString
	(dict, "__version__",
	 Py_BuildValue
	 ("s", "$Revision: 1.5 $"));
    (void)PyDict_SetItemString
	(dict, "MonthNameShort",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  MonthNameShort[0], MonthNameShort[1], MonthNameShort[2],
	  MonthNameShort[3], MonthNameShort[4], MonthNameShort[5],
	  MonthNameShort[6], MonthNameShort[7], MonthNameShort[8],
	  MonthNameShort[9], MonthNameShort[10], MonthNameShort[11],
	  MonthNameShort[12]));
    (void)PyDict_SetItemString
	(dict, "MonthNameLong",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  MonthNameLong[0], MonthNameLong[1], MonthNameLong[2],
	  MonthNameLong[3], MonthNameLong[4], MonthNameLong[5],
	  MonthNameLong[6], MonthNameLong[7], MonthNameLong[8],
	  MonthNameLong[9], MonthNameLong[10], MonthNameLong[11],
	  MonthNameLong[12]));
    (void)PyDict_SetItemString
	(dict, "JewishMonthName",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  JewishMonthName[0], JewishMonthName[1], JewishMonthName[2],
	  JewishMonthName[3], JewishMonthName[4], JewishMonthName[5],
	  JewishMonthName[6], JewishMonthName[7], JewishMonthName[8],
	  JewishMonthName[9], JewishMonthName[10], JewishMonthName[11],
	  JewishMonthName[12], JewishMonthName[13]));
    (void)PyDict_SetItemString
	(dict, "FrenchMonthName",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s,s,s,s,s,s,s,s]",
	  FrenchMonthName[0], FrenchMonthName[1], FrenchMonthName[2],
	  FrenchMonthName[3], FrenchMonthName[4], FrenchMonthName[5],
	  FrenchMonthName[6], FrenchMonthName[7], FrenchMonthName[8],
	  FrenchMonthName[9], FrenchMonthName[10], FrenchMonthName[11],
	  FrenchMonthName[12], FrenchMonthName[13]));
    (void)PyDict_SetItemString
	(dict, "DayNameShort",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s]",
	  DayNameShort[0], DayNameShort[1], DayNameShort[2],
	  DayNameShort[3], DayNameShort[4], DayNameShort[5],
	  DayNameShort[6]));
    (void)PyDict_SetItemString
	(dict, "DayNameLong",
	 Py_BuildValue
	 ("[s,s,s,s,s,s,s]",
	  DayNameLong[0], DayNameLong[1], DayNameLong[2],
	  DayNameLong[3], DayNameLong[4], DayNameLong[5],
	  DayNameLong[6]));
}
